/***************************************************************************
 *   Copyright (C) 2008 by Thomas Schildknecht <danakil@gmail.com>         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "panelspacer.h"

#include <QPainter>
#include <QGraphicsLinearLayout>
#include <KColorScheme>
#include <KConfigDialog>
#include <Plasma/Theme>
#include <Plasma/Containment>

#define MINIMALSIZE 6

PanelSpacer::PanelSpacer(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
    , m_configureMode(false)
    , m_separatorWidth(2)
    , m_containment(0)
{
    resize(220, 80);
}

void PanelSpacer::init()
{
    KConfigGroup cg = config();
    m_spacerSize = cg.readEntry("spacerSize", MINIMALSIZE + 10);
    m_separatorStyle = (SeparatorStyle)cg.readEntry("separatorStyle", (int)Line);
    m_separatorPosition = (SeparatorPosition)cg.readEntry("separatorPosition", (int)LeftOrTop);
    m_spacerStretch = cg.readEntry("separatorStretch", false);
}

void PanelSpacer::constraintsEvent(Plasma::Constraints constraints)
{
    kDebug();
    if (constraints & Plasma::FormFactorConstraint) {
        setHasConfigurationInterface(formFactor()==Plasma::Horizontal ||
                                     formFactor()==Plasma::Vertical);
        if (m_containment) {
            disconnect(m_containment, SIGNAL(appletAdded(Plasma::Applet*, const QPointF&)),
                       this, SLOT(calculateBlankSpace()));
            disconnect(m_containment, SIGNAL(appletRemoved(Plasma::Applet*)),
                       this, SLOT(calculateBlankSpace()));
        }
        m_containment = containment();
        if (m_containment) {
            connect(m_containment, SIGNAL(appletAdded(Plasma::Applet*, const QPointF&)),
                    this, SLOT(calculateBlankSpace()));
            connect(m_containment, SIGNAL(appletRemoved(Plasma::Applet*)),
                    this, SLOT(calculateBlankSpace()));
        }
    }
    if (constraints & Plasma::SizeConstraint || constraints & Plasma::FormFactorConstraint)
    {
        calculateBlankSpace();
    }
    update();
}

void PanelSpacer::paintInterface(QPainter *p,
                                 const QStyleOptionGraphicsItem *option,
                                 const QRect &contentsRect)
{
    Q_UNUSED (option);

    int seppos=0;
    KColorScheme plasmaColorTheme = KColorScheme(QPalette::Active, KColorScheme::View,
            Plasma::Theme::defaultTheme()->colorScheme());
    QColor sepcol = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);

    // warns the user if the plasmoid is on the desktop
    // and return
    if (formFactor() == Plasma::Planar || formFactor() == Plasma::MediaCenter) {
        p->setPen(QPen(sepcol));
        setHasConfigurationInterface(false);
        p->drawText(contentsRect,Qt::AlignLeft|Qt::AlignVCenter|Qt::TextWordWrap,
                i18n("I am a PanelSpacer so I am quite useless here... Please put me in a panel!"));
        return;
    }

    // draw a frame around the applet area when
    // the config dialog is displayed
    if (m_configureMode) {
        sepcol.setAlphaF(0.08);
        QBrush appletAreaB(sepcol);
        p->setPen(Qt::NoPen);
        p->setBrush(appletAreaB);

        if (formFactor() == Plasma::Horizontal) {
            if (m_spacerSize >= MINIMALSIZE) {   // minimal size area
                p->drawRect(0,0, m_spacerSize,contentsRect.bottom());
            }
            if (m_spacerStretch) {           // stretched area
                if (m_spacerSize > MINIMALSIZE) {
                    p->drawRect(m_spacerSize+4,contentsRect.top(),
                                contentsRect.right()-m_spacerSize-4,contentsRect.bottom());
                } else {
                    p->drawRect(contentsRect);
                }
            }
        }
        else if (formFactor() == Plasma::Vertical) {
            if (m_spacerSize >= MINIMALSIZE) {   // minimal size area
                p->drawRect(0,0, contentsRect.right(), m_spacerSize);
            }
            if (m_spacerStretch) {           // stretched area
                if (m_spacerSize > MINIMALSIZE) {
                    p->drawRect(0, m_spacerSize+4, contentsRect.right(),
                                contentsRect.bottom()-m_spacerSize-4);
                } else {
                    p->drawRect(contentsRect);
                }
            }
        }
    }

    sepcol.setAlphaF(0.2);
    p->setPen(QPen(sepcol));

    switch (m_separatorStyle)
    {
        case None:
            break;
        case Line:
            if (formFactor() == Plasma::Horizontal) {
                if (m_separatorPosition == Middle) {
                    seppos += ((int)contentsRect.width() / 2);
                } else if (m_separatorPosition == RightOrBottom) {
                    seppos += (int)contentsRect.width() - m_separatorWidth;
                }
                p->drawLine((int)contentsRect.left()+seppos, 0,
                            (int)contentsRect.left()+seppos, (int)contentsRect.height());
            }
            else if (formFactor() == Plasma::Vertical) {
                if (m_separatorPosition == Middle) {
                    seppos += ((int)contentsRect.height() / 2);
                } else if (m_separatorPosition == RightOrBottom) {
                    seppos += (int)contentsRect.height() - m_separatorWidth;
                }
                p->drawLine(0, (int)contentsRect.top() + seppos,
                            (int)contentsRect.width(), (int)contentsRect.top() + seppos);
            }
            break;
    }
}

void PanelSpacer::createConfigurationInterface(KConfigDialog *parent)
{
    m_configureMode = true;

    QWidget *widget = new QWidget();
    uiConfig.setupUi(widget);
    parent->setButtons(KDialog::Ok | KDialog::Cancel);
    parent->addPage(widget, parent->windowTitle(), icon()); //parent->setMainWidget(widget);

    uiConfig.horizontalSliderSize->setMinimum(MINIMALSIZE);
    uiConfig.horizontalSliderSize->setMaximum(m_maximumSize);
    uiConfig.horizontalSliderSize->setValue(m_spacerSize);

    if (m_spacerStretch) {
        uiConfig.checkBoxStretch->setChecked(true);
    } else {
        uiConfig.checkBoxStretch->setChecked(false);
    }

    uiConfig.comboBoxSeparatorStyle->addItem(i18nc("Type of visual separator", "None"));
    uiConfig.comboBoxSeparatorStyle->addItem(i18nc("Type of visual separator", "Line"));

    if (formFactor() == Plasma::Horizontal) {
        uiConfig.comboBoxSeparatorPosition->addItem(i18nc("Position of the separator", "Left"));
        uiConfig.comboBoxSeparatorPosition->addItem(i18nc("Position of the separator", "Middle"));
        uiConfig.comboBoxSeparatorPosition->addItem(i18nc("Position of the separator", "Right"));
    }
    else if (formFactor() == Plasma::Vertical){
        uiConfig.comboBoxSeparatorPosition->addItem(i18nc("Position of the separator", "Top"));
        uiConfig.comboBoxSeparatorPosition->addItem(i18nc("Position of the separator", "Middle"));
        uiConfig.comboBoxSeparatorPosition->addItem(i18nc("Position of the separator", "Bottom"));
    }

    uiConfig.comboBoxSeparatorStyle->setCurrentIndex(m_separatorStyle);
    uiConfig.comboBoxSeparatorPosition->setCurrentIndex(m_separatorPosition);

    if(m_separatorStyle == None) {
        uiConfig.comboBoxSeparatorPosition->setEnabled(false);
    }

    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(finished ()), this, SLOT(slotConfigClosed()));
    connect(uiConfig.checkBoxStretch, SIGNAL(toggled(bool)),
            this, SLOT(slotSpacerStretchChanged(bool)));
    connect(uiConfig.horizontalSliderSize, SIGNAL(sliderMoved(int)),
            this, SLOT(slotSpacerSizeChanged(int)));
    connect(uiConfig.comboBoxSeparatorStyle, SIGNAL(currentIndexChanged(int)),
            this, SLOT(slotSeparatorStyleChanged(int)));
    connect(uiConfig.comboBoxSeparatorPosition, SIGNAL(currentIndexChanged(int)),
            this, SLOT(slotSeparatorPositionChanged(int)));

    update();
}

void PanelSpacer::configAccepted()
{
    KConfigGroup cg = config();
    cg.writeEntry("spacerSize", uiConfig.horizontalSliderSize->value());
    cg.writeEntry("separatorStyle", uiConfig.comboBoxSeparatorStyle->currentIndex());
    cg.writeEntry("separatorPosition", uiConfig.comboBoxSeparatorPosition->currentIndex());
    cg.writeEntry("separatorStretch", uiConfig.checkBoxStretch->isChecked());
    emit configNeedsSaving();
}

void PanelSpacer::slotConfigClosed()
{
    m_configureMode = false;
    update();
}

void PanelSpacer::slotSpacerStretchChanged(bool checkstate)
{
    m_spacerStretch = checkstate;
    updateConstraints();
}

void PanelSpacer::slotSeparatorPositionChanged(int index)
{
    m_separatorPosition = (SeparatorPosition)index;
    updateConstraints();
}

void PanelSpacer::slotSeparatorStyleChanged(int index)
{
    m_separatorStyle = (SeparatorStyle)index;

    if (index == None) {
        uiConfig.comboBoxSeparatorPosition->setEnabled(false);
    } else {
        uiConfig.comboBoxSeparatorPosition->setEnabled(true);
    }
    updateConstraints();
}

void PanelSpacer::slotSpacerSizeChanged(int size)
{
    m_spacerSize = size;
    updateConstraints();
}

void PanelSpacer::calculateBlankSpace()
{
    m_maximumSize = 600;
    if (m_containment) {
        if (formFactor() == Plasma::Horizontal) {
            m_maximumSize = m_containment->geometry().width();
        } else {
            m_maximumSize = m_containment->geometry().height();
        }
        qreal spacing = 0;
        QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout*>(m_containment->layout());
        if (lay) {
            spacing = lay->spacing();
        }
        foreach (Plasma::Applet* applet, m_containment->applets()) {
            if (applet != this) {
                kDebug() << applet->name() << applet->preferredSize() << spacing;
                if (formFactor() == Plasma::Horizontal) {
                    m_maximumSize -= applet->preferredWidth() + spacing;
                } else {
                    m_maximumSize -= applet->preferredHeight() + spacing;
                }
            }
        }
    }
    m_spacerSize = qMin(m_spacerSize, m_maximumSize);
    if (formFactor() == Plasma::Horizontal) {
        setMinimumWidth(m_spacerSize);
        if(!m_spacerStretch) {
            setMaximumWidth(m_spacerSize);
        } else {
            setMaximumWidth(-1);
        }
    }
    else if (formFactor() == Plasma::Vertical) {
        setMinimumHeight(m_spacerSize);
        if(!m_spacerStretch) {
            setMaximumHeight(m_spacerSize);
        } else {
            setMaximumHeight(-1);
        }
    }
    kDebug() << m_spacerSize << m_maximumSize;
}

#include "panelspacer.moc"
